@@ -242,6 +242,39 @@ class TestDatabase(DatabaseTestCase):
         db.close()
         alt_db.close()
 
+    def test_bind_regression(self):
+        class Base(Model):
+            class Meta:
+                database = None
+
+        class A(Base): pass
+        class B(Base): pass
+        class AB(Base):
+            a = ForeignKeyField(A)
+            b = ForeignKeyField(B)
+
+        self.assertTrue(A._meta.database is None)
+
+        db = get_in_memory_db()
+        with db.bind_ctx([A, B]):
+            self.assertEqual(A._meta.database, db)
+            self.assertEqual(B._meta.database, db)
+            self.assertEqual(AB._meta.database, db)
+
+        self.assertTrue(A._meta.database is None)
+        self.assertTrue(B._meta.database is None)
+        self.assertTrue(AB._meta.database is None)
+
+        class C(Base):
+            a = ForeignKeyField(A)
+
+        with db.bind_ctx([C], bind_refs=False):
+            self.assertEqual(C._meta.database, db)
+            self.assertTrue(A._meta.database is None)
+
+        self.assertTrue(C._meta.database is None)
+        self.assertTrue(A._meta.database is None)
+
     def test_batch_commit(self):
         class PatchCommitDatabase(SqliteDatabase):
             commits = 0
@@ -452,8 +485,18 @@ class Note(TestModel):
         table_name = 'notes'
 
 
+class Person(TestModel):
+    first = CharField()
+    last = CharField()
+    email = CharField()
+    class Meta:
+        indexes = (
+            (('last', 'first'), False),
+        )
+
+
 class TestIntrospection(ModelTestCase):
-    requires = [Category, User, UniqueModel, IndexedModel]
+    requires = [Category, User, UniqueModel, IndexedModel, Person]
 
     def test_table_exists(self):
         self.assertTrue(self.database.table_exists(User._meta.table_name))
@@ -486,6 +529,15 @@ class TestIntrospection(ModelTestCase):
             ('indexed_model_first_last_dob', ['first', 'last', 'dob'], True,
              'indexed_model')])
 
+        # Multi-column index where columns are in different order than declared
+        # on the table.
+        indexes = self.database.get_indexes('person')
+        data = [(index.name, index.columns, index.unique)
+                for index in indexes
+                if index.name not in ('person_pkey', 'PRIMARY')]
+        self.assertEqual(data, [
+            ('person_last_first', ['last', 'first'], False)])
+
     def test_get_columns(self):
         columns = self.database.get_columns('indexed_model')
         data = [(c.name, c.null, c.primary_key, c.table)
@@ -513,7 +565,7 @@ class TestIntrospection(ModelTestCase):
     @requires_models(Note)
     def test_get_views(self):
         def normalize_view_meta(view_meta):
-            sql_ws_norm = re.sub('\n\s+', ' ', view_meta.sql.strip('; '))
+            sql_ws_norm = re.sub(r'[\n\s]+', ' ', view_meta.sql.strip('; '))
             return view_meta.name, (sql_ws_norm
                                     .replace('`peewee_test`.', '')
                                     .replace('`notes`.', '')
