@@ -71,6 +71,14 @@ class TestSelectQuery(BaseTestCase):
                 'WHERE ("t1"."name" IN (?, ?))'))
             self.assertEqual(sorted(params), ['charlie', 'huey'])
 
+        query = (Person
+                 .select()
+                 .where(Person.id.in_(range(1, 10, 2))))
+        self.assertSQL(query, (
+            'SELECT "t1"."id", "t1"."name", "t1"."dob" '
+            'FROM "person" AS "t1" '
+            'WHERE ("t1"."id" IN (?, ?, ?, ?, ?))'), [1, 3, 5, 7, 9])
+
     def test_select_subselect_function(self):
         # For functions whose only argument is a subquery, we do not need to
         # include additional parentheses -- in fact, some databases will report
@@ -232,6 +240,29 @@ class TestSelectQuery(BaseTestCase):
             'WHERE (("t1"."dob" > ?) OR ("t1"."dob" < ?))'),
             [datetime.date(1980, 1, 1), datetime.date(1950, 1, 1)])
 
+    def test_limit(self):
+        base = User.select(User.c.id)
+        self.assertSQL(base.limit(None), (
+            'SELECT "t1"."id" FROM "users" AS "t1"'), [])
+        self.assertSQL(base.limit(10), (
+            'SELECT "t1"."id" FROM "users" AS "t1" LIMIT ?'), [10])
+        self.assertSQL(base.limit(10).offset(3), (
+            'SELECT "t1"."id" FROM "users" AS "t1" '
+            'LIMIT ? OFFSET ?'), [10, 3])
+        self.assertSQL(base.limit(0), (
+            'SELECT "t1"."id" FROM "users" AS "t1" LIMIT ?'), [0])
+
+        self.assertSQL(base.offset(3), (
+            'SELECT "t1"."id" FROM "users" AS "t1" OFFSET ?'), [3],
+            limit_max=None)
+        # Some databases do not support offset without corresponding LIMIT:
+        self.assertSQL(base.offset(3), (
+            'SELECT "t1"."id" FROM "users" AS "t1" LIMIT ? OFFSET ?'), [-1, 3],
+            limit_max=-1)
+        self.assertSQL(base.limit(0).offset(3), (
+            'SELECT "t1"."id" FROM "users" AS "t1" LIMIT ? OFFSET ?'), [0, 3],
+            limit_max=-1)
+
     def test_simple_join(self):
         query = (User
                  .select(
@@ -361,18 +392,22 @@ class TestSelectQuery(BaseTestCase):
             q1.c.next_fib_n,
             q1.c.fib_n + q1.c.next_fib_n)).from_(q1).where(n < 10)
 
-        cte = q1.union_all(rterm)
-        query = cte.select_from(cte.c.n, cte.c.fib_n)
-        self.assertSQL(query, (
-            'WITH RECURSIVE "fibonacci" AS ('
-            'SELECT ? AS "n", ? AS "fib_n", ? AS "next_fib_n" '
-            'UNION ALL '
-            'SELECT ("fibonacci"."n" + ?) AS "n", "fibonacci"."next_fib_n", '
-            '("fibonacci"."fib_n" + "fibonacci"."next_fib_n") '
-            'FROM "fibonacci" '
-            'WHERE ("n" < ?)) '
-            'SELECT "fibonacci"."n", "fibonacci"."fib_n" '
-            'FROM "fibonacci"'), [1, 0, 1, 1, 10])
+        cases = (
+            (q1.union_all, 'UNION ALL'),
+            (q1.union, 'UNION'))
+        for method, clause in cases:
+            cte = method(rterm)
+            query = cte.select_from(cte.c.n, cte.c.fib_n)
+            self.assertSQL(query, (
+                'WITH RECURSIVE "fibonacci" AS ('
+                'SELECT ? AS "n", ? AS "fib_n", ? AS "next_fib_n" '
+                '%s '
+                'SELECT ("fibonacci"."n" + ?) AS "n", "fibonacci"."next_fib_n", '
+                '("fibonacci"."fib_n" + "fibonacci"."next_fib_n") '
+                'FROM "fibonacci" '
+                'WHERE ("n" < ?)) '
+                'SELECT "fibonacci"."n", "fibonacci"."fib_n" '
+                'FROM "fibonacci"' % clause), [1, 0, 1, 1, 10])
 
     def test_cte_with_count(self):
         cte = User.select(User.c.id).cte('user_ids')
@@ -770,6 +805,36 @@ class TestSelectQuery(BaseTestCase):
             'SELECT "t1"."id", "t1"."content", "t1"."user_id" '
             'FROM "notes" AS "t1" WHERE ("t1"."user_id" IS ?)'), [None])
 
+    def test_like_escape(self):
+        T = Table('tbl', ('key',))
+        def assertLike(expr, expected):
+            query = T.select().where(expr)
+            sql, params = __sql__(T.select().where(expr))
+            match_obj = re.search(r'\("t1"."key" (ILIKE[^\)]+)\)', sql)
+            if match_obj is None:
+                raise AssertionError('LIKE expression not found in query.')
+            like, = match_obj.groups()
+            self.assertEqual((like, params), expected)
+
+        cases = (
+            (T.key.contains('base'), ('ILIKE ?', ['%base%'])),
+            (T.key.contains('x_y'), ("ILIKE ? ESCAPE ?", ['%x\\_y%', '\\'])),
+            (T.key.contains('__y'), ("ILIKE ? ESCAPE ?", ['%\\_\\_y%', '\\'])),
+            (T.key.contains('%'), ("ILIKE ? ESCAPE ?", ['%\\%%', '\\'])),
+            (T.key.contains('_%'), ("ILIKE ? ESCAPE ?", ['%\\_\\%%', '\\'])),
+            (T.key.startswith('base'), ("ILIKE ?", ['base%'])),
+            (T.key.startswith('x_y'), ("ILIKE ? ESCAPE ?", ['x\\_y%', '\\'])),
+            (T.key.startswith('x%'), ("ILIKE ? ESCAPE ?", ['x\\%%', '\\'])),
+            (T.key.startswith('_%'), ("ILIKE ? ESCAPE ?", ['\\_\\%%', '\\'])),
+            (T.key.endswith('base'), ("ILIKE ?", ['%base'])),
+            (T.key.endswith('x_y'), ("ILIKE ? ESCAPE ?", ['%x\\_y', '\\'])),
+            (T.key.endswith('x%'), ("ILIKE ? ESCAPE ?", ['%x\\%', '\\'])),
+            (T.key.endswith('_%'), ("ILIKE ? ESCAPE ?", ['%\\_\\%', '\\'])),
+        )
+
+        for expr, expected in cases:
+            assertLike(expr, expected)
+
 
 class TestInsertQuery(BaseTestCase):
     def test_insert_simple(self):
@@ -1594,6 +1659,29 @@ class TestSelectFeatures(BaseTestCase):
             'FILTER (WHERE ("t1"."dob" < ?)) '
             'FROM "person" AS "t1"'), [datetime.date(2000, 1, 1)])
 
+    def test_ordered_aggregate(self):
+        agg = fn.array_agg(Person.name).order_by(Person.id.desc())
+        self.assertSQL(Person.select(agg.alias('names')), (
+            'SELECT array_agg("t1"."name" ORDER BY "t1"."id" DESC) AS "names" '
+            'FROM "person" AS "t1"'), [])
+
+        agg = fn.string_agg(Person.name, ',').order_by(Person.dob, Person.id)
+        self.assertSQL(Person.select(agg), (
+            'SELECT string_agg("t1"."name", ? ORDER BY "t1"."dob", "t1"."id")'
+            ' FROM "person" AS "t1"'), [','])
+
+        agg = (fn.string_agg(Person.name.concat('-x'), ',')
+               .order_by(Person.name.desc(), Person.dob.asc()))
+        self.assertSQL(Person.select(agg), (
+            'SELECT string_agg(("t1"."name" || ?), ? ORDER BY "t1"."name" DESC'
+            ', "t1"."dob" ASC) '
+            'FROM "person" AS "t1"'), ['-x', ','])
+
+        agg = agg.order_by()
+        self.assertSQL(Person.select(agg), (
+            'SELECT string_agg(("t1"."name" || ?), ?) '
+            'FROM "person" AS "t1"'), ['-x', ','])
+
     def test_for_update(self):
         query = (Person
                  .select()
@@ -1679,6 +1767,17 @@ class TestSelectFeatures(BaseTestCase):
             'WHERE ("t2"."username" = "t1"."name"))'), [])
 
 
+class TestExpressionSQL(BaseTestCase):
+    def test_parentheses_functions(self):
+        expr = (User.c.income + 100)
+        expr2 = expr * expr
+        query = User.select(fn.sum(expr), fn.avg(expr2))
+        self.assertSQL(query, (
+            'SELECT sum("t1"."income" + ?), '
+            'avg(("t1"."income" + ?) * ("t1"."income" + ?)) '
+            'FROM "users" AS "t1"'), [100, 100, 100])
+
+
 #Person = Table('person', ['id', 'name', 'dob'])
 
 class TestOnConflictSqlite(BaseTestCase):
